@@ -1,7 +1,7 @@
 /*
  * Please refer to the documentation hosted at
  *
- *   https://docs.rs/blazesym-c/0.1.5
+ *   https://docs.rs/blazesym-c/0.1.6
  */
 
 
@@ -69,7 +69,7 @@ typedef int16_t blaze_err;
  */
 #define BLAZE_ERR_WRITE_ZERO -257
 /**
- * An error returned when an operation ould not be completed
+ * An error returned when an operation could not be completed
  * because an "end of file" was reached prematurely.
  */
 #define BLAZE_ERR_UNEXPECTED_EOF -258
@@ -277,6 +277,10 @@ typedef uint8_t blaze_normalize_reason;
  * The address could not be found in the symbolization source.
  */
 #define BLAZE_NORMALIZE_REASON_UNKNOWN_ADDR 5
+/**
+ * An error prevented the symbolization of the address from succeeding.
+ */
+#define BLAZE_NORMALIZE_REASON_IGNORED_ERROR 7
 
 /**
  * The valid variant kind in [`blaze_user_meta`].
@@ -443,6 +447,10 @@ typedef uint8_t blaze_symbolize_reason;
  * The address belonged to an entity that is currently unsupported.
  */
 #define BLAZE_SYMBOLIZE_REASON_UNSUPPORTED 6
+/**
+ * An error prevented the symbolization of the address from succeeding.
+ */
+#define BLAZE_SYMBOLIZE_REASON_IGNORED_ERROR 7
 
 /**
  * The result of symbolization of an address.
@@ -1397,7 +1405,7 @@ void blaze_symbolize_cache_process(blaze_symbolizer *symbolizer,
  * # Safety
  * - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
  * - `src` needs to point to a valid [`blaze_symbolize_src_process`] object
- * - `abs_addrs` point to an array of `abs_addr_cnt` addresses
+ * - `abs_addrs` needs to point to an array of `abs_addr_cnt` addresses
  */
 const struct blaze_syms *blaze_symbolize_process_abs_addrs(blaze_symbolizer *symbolizer,
                                                            const struct blaze_symbolize_src_process *src,
@@ -1419,7 +1427,7 @@ const struct blaze_syms *blaze_symbolize_process_abs_addrs(blaze_symbolizer *sym
  * # Safety
  * - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
  * - `src` needs to point to a valid [`blaze_symbolize_src_kernel`] object
- * - `abs_addrs` point to an array of `abs_addr_cnt` addresses
+ * - `abs_addrs` needs to point to an array of `abs_addr_cnt` addresses
  */
 const struct blaze_syms *blaze_symbolize_kernel_abs_addrs(blaze_symbolizer *symbolizer,
                                                           const struct blaze_symbolize_src_kernel *src,
@@ -1441,7 +1449,7 @@ const struct blaze_syms *blaze_symbolize_kernel_abs_addrs(blaze_symbolizer *symb
  * # Safety
  * - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
  * - `src` needs to point to a valid [`blaze_symbolize_src_elf`] object
- * - `virt_offsets` point to an array of `virt_offset_cnt` addresses
+ * - `virt_offsets` needs to point to an array of `virt_offset_cnt` addresses
  */
 const struct blaze_syms *blaze_symbolize_elf_virt_offsets(blaze_symbolizer *symbolizer,
                                                           const struct blaze_symbolize_src_elf *src,
@@ -1463,7 +1471,7 @@ const struct blaze_syms *blaze_symbolize_elf_virt_offsets(blaze_symbolizer *symb
  * # Safety
  * - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
  * - `src` needs to point to a valid [`blaze_symbolize_src_elf`] object
- * - `file_offsets` point to an array of `file_offset_cnt` addresses
+ * - `file_offsets` needs to point to an array of `file_offset_cnt` addresses
  */
 const struct blaze_syms *blaze_symbolize_elf_file_offsets(blaze_symbolizer *symbolizer,
                                                           const struct blaze_symbolize_src_elf *src,
@@ -1485,7 +1493,7 @@ const struct blaze_syms *blaze_symbolize_elf_file_offsets(blaze_symbolizer *symb
  * # Safety
  * - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
  * - `src` needs to point to a valid [`blaze_symbolize_src_gsym_data`] object
- * - `virt_offsets` point to an array of `virt_offset_cnt` addresses
+ * - `virt_offsets` needs to point to an array of `virt_offset_cnt` addresses
  */
 const struct blaze_syms *blaze_symbolize_gsym_data_virt_offsets(blaze_symbolizer *symbolizer,
                                                                 const struct blaze_symbolize_src_gsym_data *src,
@@ -1507,7 +1515,7 @@ const struct blaze_syms *blaze_symbolize_gsym_data_virt_offsets(blaze_symbolizer
  * # Safety
  * - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
  * - `src` needs to point to a valid [`blaze_symbolize_src_gsym_file`] object
- * - `virt_offsets` point to an array of `virt_offset_cnt` addresses
+ * - `virt_offsets` needs to point to an array of `virt_offset_cnt` addresses
  */
 const struct blaze_syms *blaze_symbolize_gsym_file_virt_offsets(blaze_symbolizer *symbolizer,
                                                                 const struct blaze_symbolize_src_gsym_file *src,
