@@ -14,7 +14,6 @@ use std::os::raw::c_char;
 use std::os::unix::ffi::OsStringExt as _;
 use std::path::PathBuf;
 use std::ptr;
-use std::slice;
 
 use blazesym::normalize::Apk;
 use blazesym::normalize::Elf;
@@ -416,7 +415,7 @@ impl blaze_user_meta_elf {
             )),
             build_id: (!build_id.is_null()).then(|| unsafe {
                 Cow::Owned(
-                    Box::<[u8]>::from_raw(slice::from_raw_parts_mut(build_id, build_id_len))
+                    Box::<[u8]>::from_raw(ptr::slice_from_raw_parts_mut(build_id, build_id_len))
                         .into_vec(),
                 )
             }),
@@ -499,6 +498,8 @@ impl blaze_normalize_reason {
     pub const MISSING_SYMS: blaze_normalize_reason = blaze_normalize_reason(4);
     /// The address could not be found in the symbolization source.
     pub const UNKNOWN_ADDR: blaze_normalize_reason = blaze_normalize_reason(5);
+    /// An error prevented the symbolization of the address from succeeding.
+    pub const IGNORED_ERROR: blaze_normalize_reason = blaze_normalize_reason(7);
 }
 
 impl From<Reason> for blaze_normalize_reason {
@@ -510,6 +511,7 @@ impl From<Reason> for blaze_normalize_reason {
             Reason::InvalidFileOffset => blaze_normalize_reason::INVALID_FILE_OFFSET,
             Reason::MissingSyms => blaze_normalize_reason::MISSING_SYMS,
             Reason::UnknownAddr => blaze_normalize_reason::UNKNOWN_ADDR,
+            Reason::IgnoredError => blaze_normalize_reason::IGNORED_ERROR,
             _ => unreachable!(),
         }
     }
@@ -530,6 +532,7 @@ pub extern "C" fn blaze_normalize_reason_str(reason: blaze_normalize_reason) ->
         }
         blaze_normalize_reason::MISSING_SYMS => Reason::MissingSyms.as_bytes().as_ptr().cast(),
         blaze_normalize_reason::UNKNOWN_ADDR => Reason::UnknownAddr.as_bytes().as_ptr().cast(),
+        blaze_normalize_reason::IGNORED_ERROR => Reason::IgnoredError.as_bytes().as_ptr().cast(),
         _ => b"unknown reason\0".as_ptr().cast(),
     }
 }
@@ -840,14 +843,14 @@ pub unsafe extern "C" fn blaze_user_output_free(output: *mut blaze_normalized_us
     //         our blessed functions.
     let user_output = unsafe { Box::from_raw(output) };
     let addr_metas = unsafe {
-        Box::<[blaze_user_meta]>::from_raw(slice::from_raw_parts_mut(
+        Box::<[blaze_user_meta]>::from_raw(ptr::slice_from_raw_parts_mut(
             user_output.metas,
             user_output.meta_cnt,
         ))
     }
     .into_vec();
     let _norm_addrs = unsafe {
-        Box::<[blaze_normalized_output]>::from_raw(slice::from_raw_parts_mut(
+        Box::<[blaze_normalized_output]>::from_raw(ptr::slice_from_raw_parts_mut(
             user_output.outputs,
             user_output.output_cnt,
         ))
@@ -983,6 +986,7 @@ mod tests {
             ),
             (Reason::MissingSyms, blaze_normalize_reason::MISSING_SYMS),
             (Reason::UnknownAddr, blaze_normalize_reason::UNKNOWN_ADDR),
+            (Reason::IgnoredError, blaze_normalize_reason::IGNORED_ERROR),
         ];
 
         for (reason, expected) in data {
