@@ -480,6 +480,8 @@ impl blaze_symbolize_reason {
     pub const UNKNOWN_ADDR: blaze_symbolize_reason = blaze_symbolize_reason(5);
     /// The address belonged to an entity that is currently unsupported.
     pub const UNSUPPORTED: blaze_symbolize_reason = blaze_symbolize_reason(6);
+    /// An error prevented the symbolization of the address from succeeding.
+    pub const IGNORED_ERROR: blaze_symbolize_reason = blaze_symbolize_reason(7);
 }
 
 
@@ -492,6 +494,7 @@ impl From<Reason> for blaze_symbolize_reason {
             Reason::MissingSyms => blaze_symbolize_reason::MISSING_SYMS,
             Reason::Unsupported => blaze_symbolize_reason::UNSUPPORTED,
             Reason::UnknownAddr => blaze_symbolize_reason::UNKNOWN_ADDR,
+            Reason::IgnoredError => blaze_symbolize_reason::IGNORED_ERROR,
             _ => unreachable!(),
         }
     }
@@ -514,6 +517,7 @@ pub extern "C" fn blaze_symbolize_reason_str(reason: blaze_symbolize_reason) ->
         blaze_symbolize_reason::MISSING_SYMS => Reason::MissingSyms.as_bytes().as_ptr().cast(),
         blaze_symbolize_reason::UNKNOWN_ADDR => Reason::UnknownAddr.as_bytes().as_ptr().cast(),
         blaze_symbolize_reason::UNSUPPORTED => Reason::Unsupported.as_bytes().as_ptr().cast(),
+        blaze_symbolize_reason::IGNORED_ERROR => Reason::IgnoredError.as_bytes().as_ptr().cast(),
         _ => b"unknown reason\0".as_ptr().cast(),
     }
 }
@@ -1084,7 +1088,7 @@ unsafe fn blaze_symbolize_impl(
 /// # Safety
 /// - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
 /// - `src` needs to point to a valid [`blaze_symbolize_src_process`] object
-/// - `abs_addrs` point to an array of `abs_addr_cnt` addresses
+/// - `abs_addrs` needs to point to an array of `abs_addr_cnt` addresses
 #[no_mangle]
 pub unsafe extern "C" fn blaze_symbolize_process_abs_addrs(
     symbolizer: *mut blaze_symbolizer,
@@ -1117,7 +1121,7 @@ pub unsafe extern "C" fn blaze_symbolize_process_abs_addrs(
 /// # Safety
 /// - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
 /// - `src` needs to point to a valid [`blaze_symbolize_src_kernel`] object
-/// - `abs_addrs` point to an array of `abs_addr_cnt` addresses
+/// - `abs_addrs` needs to point to an array of `abs_addr_cnt` addresses
 #[no_mangle]
 pub unsafe extern "C" fn blaze_symbolize_kernel_abs_addrs(
     symbolizer: *mut blaze_symbolizer,
@@ -1150,7 +1154,7 @@ pub unsafe extern "C" fn blaze_symbolize_kernel_abs_addrs(
 /// # Safety
 /// - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
 /// - `src` needs to point to a valid [`blaze_symbolize_src_elf`] object
-/// - `virt_offsets` point to an array of `virt_offset_cnt` addresses
+/// - `virt_offsets` needs to point to an array of `virt_offset_cnt` addresses
 #[no_mangle]
 pub unsafe extern "C" fn blaze_symbolize_elf_virt_offsets(
     symbolizer: *mut blaze_symbolizer,
@@ -1189,7 +1193,7 @@ pub unsafe extern "C" fn blaze_symbolize_elf_virt_offsets(
 /// # Safety
 /// - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
 /// - `src` needs to point to a valid [`blaze_symbolize_src_elf`] object
-/// - `file_offsets` point to an array of `file_offset_cnt` addresses
+/// - `file_offsets` needs to point to an array of `file_offset_cnt` addresses
 #[no_mangle]
 pub unsafe extern "C" fn blaze_symbolize_elf_file_offsets(
     symbolizer: *mut blaze_symbolizer,
@@ -1229,7 +1233,7 @@ pub unsafe extern "C" fn blaze_symbolize_elf_file_offsets(
 /// # Safety
 /// - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
 /// - `src` needs to point to a valid [`blaze_symbolize_src_gsym_data`] object
-/// - `virt_offsets` point to an array of `virt_offset_cnt` addresses
+/// - `virt_offsets` needs to point to an array of `virt_offset_cnt` addresses
 #[no_mangle]
 pub unsafe extern "C" fn blaze_symbolize_gsym_data_virt_offsets(
     symbolizer: *mut blaze_symbolizer,
@@ -1268,7 +1272,7 @@ pub unsafe extern "C" fn blaze_symbolize_gsym_data_virt_offsets(
 /// # Safety
 /// - `symbolizer` needs to point to a valid [`blaze_symbolizer`] object
 /// - `src` needs to point to a valid [`blaze_symbolize_src_gsym_file`] object
-/// - `virt_offsets` point to an array of `virt_offset_cnt` addresses
+/// - `virt_offsets` needs to point to an array of `virt_offset_cnt` addresses
 #[no_mangle]
 pub unsafe extern "C" fn blaze_symbolize_gsym_file_virt_offsets(
     symbolizer: *mut blaze_symbolizer,
@@ -1482,6 +1486,7 @@ mod tests {
             (Reason::MissingSyms, blaze_symbolize_reason::MISSING_SYMS),
             (Reason::Unsupported, blaze_symbolize_reason::UNSUPPORTED),
             (Reason::UnknownAddr, blaze_symbolize_reason::UNKNOWN_ADDR),
+            (Reason::IgnoredError, blaze_symbolize_reason::IGNORED_ERROR),
         ];
 
         for (reason, expected) in data {
