@@ -5,6 +5,7 @@ use std::ffi::OsStr;
 use std::fmt::Debug;
 use std::fmt::Formatter;
 use std::fmt::Result as FmtResult;
+use std::io;
 use std::mem;
 use std::mem::swap;
 use std::ops::ControlFlow;
@@ -19,9 +20,11 @@ use gimli::Dwarf;
 use crate::dwarf::reader::Endianess;
 use crate::dwarf::reader::R;
 use crate::elf::ElfParser;
+use crate::elf::ElfResolverData;
 #[cfg(test)]
 use crate::elf::DEFAULT_DEBUG_DIRS;
 use crate::error::IntoCowStr;
+use crate::file_cache::FileCache;
 use crate::inspect::FindAddrOpts;
 use crate::inspect::ForEachFn;
 use crate::inspect::Inspect;
@@ -40,7 +43,6 @@ use crate::Addr;
 use crate::Error;
 use crate::ErrorExt;
 use crate::ErrorKind;
-use crate::Mmap;
 use crate::Result;
 use crate::SymType;
 
@@ -92,7 +94,20 @@ impl From<Option<gimli::DwLang>> for SrcLang {
 }
 
 
-/// Find a debug file in a list of default directories.
+fn try_canonicalize(path: &Path) -> io::Result<Cow<'_, Path>> {
+    match path.canonicalize() {
+        Ok(path) => Ok(Cow::Owned(path)),
+        Err(err) if err.kind() == io::ErrorKind::NotFound => {
+            // If the file doesn't actually exist we may still want to use
+            // its path for constructing *.debug candidates, so be liberal
+            // here and just ignore this error.
+            Ok(Cow::Borrowed(path))
+        }
+        Err(err) => Err(err),
+    }
+}
+
+/// Find a debug file in a list of directories.
 ///
 /// `linker` is the path to the file containing the debug link. This function
 /// searches a couple of "well-known" locations and then others constructed
@@ -101,9 +116,8 @@ impl From<Option<gimli::DwLang>> for SrcLang {
 /// # Notes
 /// This function ignores any errors encountered.
 fn find_debug_file(file: &OsStr, linker: Option<&Path>, debug_dirs: &[PathBuf]) -> Option<PathBuf> {
-    let canonical_linker = linker.and_then(|linker| linker.canonicalize().ok());
+    let canonical_linker = linker.and_then(|linker| try_canonicalize(linker).ok());
     let it = DebugFileIter::new(debug_dirs, canonical_linker.as_deref(), file);
-
     for path in it {
         if path.exists() {
             debug!("found debug info at `{}`", path.display());
@@ -118,12 +132,10 @@ fn find_debug_file(file: &OsStr, linker: Option<&Path>, debug_dirs: &[PathBuf])
 }
 
 
-// TODO: We really should have a cache of debug link targets as well, to
-//       avoid potentially re-parsing the same files over and over
-//       again.
 fn try_deref_debug_link(
     parser: &ElfParser,
     debug_dirs: &[PathBuf],
+    elf_cache: Option<&FileCache<ElfResolverData>>,
 ) -> Result<Option<Rc<ElfParser>>> {
     if let Some((file, checksum)) = read_debug_link(parser)? {
         // TODO: Usage of the module here is fishy, as it may not
@@ -133,10 +145,25 @@ fn try_deref_debug_link(
         let linker = parser.module().map(OsStr::as_ref);
         match find_debug_file(file, linker, debug_dirs) {
             Some(path) => {
-                let mmap = Mmap::builder().open(&path).with_context(|| {
-                    format!("failed to open debug link destination `{}`", path.display())
-                })?;
-                let crc = debug_link_crc32(&mmap);
+                let tmp_parser;
+                let dst_parser = if let Some(elf_cache) = elf_cache {
+                    // TODO: Unclear whether we should provide `debug_dirs`
+                    //       here instead of `None`?
+                    elf_cache
+                        .elf_resolver(&path, None)
+                        .with_context(|| {
+                            format!("failed to open debug link destination `{}`", path.display())
+                        })?
+                        .parser()
+                } else {
+                    let parser = ElfParser::open(&path).with_context(|| {
+                        format!("failed to open debug link destination `{}`", path.display())
+                    })?;
+                    tmp_parser = Rc::new(parser);
+                    &tmp_parser
+                };
+                let mmap = dst_parser.backend();
+                let crc = debug_link_crc32(mmap);
                 if crc != checksum {
                     return Err(Error::with_invalid_data(format!(
                         "debug link destination `{}` checksum does not match \
@@ -144,10 +171,7 @@ fn try_deref_debug_link(
                         path.display()
                     )))
                 }
-
-                let module = path.into_os_string();
-                let dst_parser = Rc::new(ElfParser::from_mmap(mmap, Some(module)));
-                Ok(Some(dst_parser))
+                Ok(Some(Rc::clone(dst_parser)))
             }
             None => Ok(None),
         }
@@ -159,16 +183,27 @@ fn try_deref_debug_link(
 
 /// Try to find a DWARF package (`.dwp`) "belonging" to the file
 /// referenced by the given [`ElfParser`].
-fn try_find_dwp(parser: &ElfParser) -> Result<Option<Rc<ElfParser>>> {
+fn try_find_dwp(
+    parser: &ElfParser,
+    elf_cache: Option<&FileCache<ElfResolverData>>,
+) -> Result<Option<Rc<ElfParser>>> {
     if let Some(path) = parser.module() {
         let mut dwp_path = path.to_os_string();
         let () = dwp_path.push(".dwp");
         let dwp_path = PathBuf::from(dwp_path);
 
-        match ElfParser::open(&dwp_path) {
+        let result = if let Some(elf_cache) = elf_cache {
+            elf_cache
+                .elf_resolver(&dwp_path, None)
+                .map(|resolver| Rc::clone(resolver.parser()))
+        } else {
+            ElfParser::open(&dwp_path).map(Rc::new)
+        };
+
+        match result {
             Ok(parser) => {
                 log::debug!("using DWARF package `{}`", dwp_path.display());
-                Ok(Some(Rc::new(parser)))
+                Ok(Some(parser))
             }
             Err(err) if err.kind() == ErrorKind::NotFound => Ok(None),
             Err(err) => Err(err),
@@ -203,9 +238,13 @@ impl DwarfResolver {
         &self.parser
     }
 
-    pub(crate) fn from_parser(parser: Rc<ElfParser>, debug_dirs: &[PathBuf]) -> Result<Self> {
-        let linkee_parser = try_deref_debug_link(&parser, debug_dirs)?;
-        let dwp_parser = try_find_dwp(&parser)?;
+    pub(crate) fn from_parser(
+        parser: Rc<ElfParser>,
+        debug_dirs: &[PathBuf],
+        elf_cache: Option<&FileCache<ElfResolverData>>,
+    ) -> Result<Self> {
+        let linkee_parser = try_deref_debug_link(&parser, debug_dirs, elf_cache)?;
+        let dwp_parser = try_find_dwp(&parser, elf_cache)?;
 
         // SAFETY: We own the `ElfParser` and make sure that it stays
         //         around while the `Units` object uses it. As such, it
@@ -259,7 +298,7 @@ impl DwarfResolver {
             .iter()
             .map(PathBuf::from)
             .collect::<Vec<_>>();
-        Self::from_parser(Rc::new(parser), debug_dirs.as_slice())
+        Self::from_parser(Rc::new(parser), debug_dirs.as_slice(), None)
     }
 
     /// Try converting a `Function` into a `SymInfo`.
@@ -526,6 +565,8 @@ mod tests {
     use std::ops::ControlFlow;
     use std::path::PathBuf;
 
+    use tempfile::NamedTempFile;
+
     use test_log::test;
 
     use crate::ErrorKind;
@@ -554,6 +595,22 @@ mod tests {
         assert_eq!(format!("{err:#}"), format!("failed to read: {inner}"));
     }
 
+    /// Make sure that `try_canonicalize` works as it should.
+    #[test]
+    fn canonicalization_attempt() {
+        let file = NamedTempFile::new().unwrap();
+        let path = file.path().to_path_buf();
+        let expected = path.clone();
+
+        let canonical = try_canonicalize(&path).unwrap();
+        assert_eq!(canonical, expected);
+
+        drop(file);
+
+        let canonical = try_canonicalize(&path).unwrap();
+        assert_eq!(canonical, expected);
+    }
+
     /// Check that we resolve debug links correctly.
     #[test]
     fn debug_link_resolution() {
@@ -579,14 +636,14 @@ mod tests {
             .join("data")
             .join("test-rs-split-dwarf.bin");
         let parser = ElfParser::open(&path).unwrap();
-        let dwp_parser = try_find_dwp(&parser).unwrap();
+        let dwp_parser = try_find_dwp(&parser, None).unwrap();
         assert!(dwp_parser.is_some());
 
         let path = Path::new(&env!("CARGO_MANIFEST_DIR"))
             .join("data")
             .join("test-rs.bin");
         let parser = ElfParser::open(&path).unwrap();
-        let dwp_parser = try_find_dwp(&parser).unwrap();
+        let dwp_parser = try_find_dwp(&parser, None).unwrap();
         assert!(dwp_parser.is_none());
     }
 
