@@ -525,13 +525,14 @@ where
                             (chdr.ch_type, chdr.ch_size)
                         };
 
-                        let decompressed = match ch_type {
+                        let mut decompressed = match ch_type {
                             t if t == ELFCOMPRESS_ZLIB => decompress_zlib(data),
                             t if t == ELFCOMPRESS_ZSTD => decompress_zstd(data),
                             _ => Err(Error::with_unsupported(format!(
                                 "ELF section is compressed with unknown compression algorithm ({ch_type})",
                             ))),
                         }?;
+                        decompressed.shrink_to_fit();
                         debug_assert_eq!(
                             decompressed.len(),
                             ch_size as usize,
@@ -1017,7 +1018,7 @@ impl ElfParser<File> {
         open_impl(path, module)
     }
 
-    /// Retrieve a reference to the backend in use.
+    /// Retrieve a reference to the file backend in use.
     pub(crate) fn backend(&self) -> &File {
         &self._backend
     }
@@ -1062,6 +1063,11 @@ impl ElfParser<Mmap> {
         let path = path.actual_path();
         open_impl(path, module)
     }
+
+    /// Retrieve a reference to the `Mmap` backend in use.
+    pub(crate) fn backend(&self) -> &Mmap {
+        &self._backend
+    }
 }
 
 impl ElfParser<StaticMem> {
