@@ -166,8 +166,8 @@ impl ErrorImpl {
             Self::Dwarf { backtrace, .. } => Some(backtrace),
             Self::Io { backtrace, .. } => Some(backtrace),
             Self::Std { backtrace, .. } => Some(backtrace),
-            Self::ContextOwned { .. } => None,
-            Self::ContextStatic { .. } => None,
+            Self::ContextOwned { source, .. } => source.backtrace(),
+            Self::ContextStatic { source, .. } => source.backtrace(),
         }
     }
 
@@ -886,6 +886,13 @@ Caused by:
         let start_idx = debug.find("Stack backtrace").unwrap();
         let backtrace = &debug[start_idx..];
         assert!(backtrace.contains("src/error.rs"), "{backtrace}");
+
+        let err = err.context("foobar");
+        let debug = format!("{err:?}");
+
+        let start_idx = debug.find("Stack backtrace").unwrap();
+        let backtrace = &debug[start_idx..];
+        assert!(backtrace.contains("src/error.rs"), "{backtrace}");
     }
 
     /// Make sure that we do not emit backtraces in errors when
