@@ -38,7 +38,8 @@ pub struct EntryPath {
     /// `/proc/<xxx>/map_files/` component.
     ///
     /// This path should generally be used on the local system, unless perhaps
-    /// for reporting purposes (for which `path` below may be more appropriate).
+    /// for reporting purposes (for which [`symbolic_path`][Self::symbolic_path]
+    /// may be more appropriate).
     pub maps_file: PathBuf,
     /// The path to the file backing the proc maps entry as found directly in
     /// the `/proc/<xxx>/maps` file. This path should generally only be used for
@@ -72,6 +73,8 @@ pub enum PathName {
     /// The member is a non-file system backed entity.
     ///
     /// Examples include the heap (`[heap]`) or BPF programs, for example.
+    // TODO: Consider making the inner variant an enum to represent
+    //       entities such as `[vdso]` in a more type-safe manner.
     Component(String),
 }
 
@@ -204,7 +207,7 @@ pub(crate) fn parse_path_name(
             let symbolic_path =
                 bytes_to_path(path.strip_suffix(b" (deleted)").unwrap_or(path))?.to_path_buf();
             // Make sure to resolve the potentially symbolic PID so that
-            // it have more of a meaning in case of remote symbolization
+            // it has more of a meaning in case of remote symbolization
             // use cases.
             let pid = pid.resolve();
             let maps_file =
@@ -397,15 +400,7 @@ pub(crate) fn filter_relevant(entry: &MapsEntry) -> bool {
     // Only readable (r---) or executable (--x-) entries are of relevance.
     // NB: Please keep this logic in sync with flags being used by
     //     `procmap_query`.
-    if (entry.perm & (Perm::R | Perm::X)) == Perm::default() {
-        return false
-    }
-
-    match entry.path_name {
-        Some(PathName::Path(..)) => true,
-        Some(PathName::Component(..)) => false,
-        None => true,
-    }
+    (entry.perm & (Perm::R | Perm::X)) != Perm::default()
 }
 
 /// Parse the maps file for the process with the given PID and make sure
