@@ -28,9 +28,8 @@ use crate::file_cache::FileCache;
 #[cfg(feature = "gsym")]
 use crate::gsym::GsymResolver;
 use crate::insert_map::InsertMap;
+use crate::kernel::KernelCache;
 use crate::kernel::KernelResolver;
-use crate::kernel::KsymResolver;
-use crate::kernel::KALLSYMS;
 use crate::log;
 use crate::maps;
 use crate::maps::EntryPath;
@@ -46,8 +45,6 @@ use crate::perf_map::PerfMap;
 use crate::symbolize::Resolve;
 use crate::symbolize::TranslateFileOffset;
 use crate::util;
-#[cfg(linux)]
-use crate::util::uname_release;
 use crate::util::Dbg;
 #[cfg(feature = "tracing")]
 use crate::util::Hexify;
@@ -132,14 +129,14 @@ fn maybe_demangle_impl(name: Cow<'_, str>, language: SrcLang) -> Cow<'_, str> {
             .map(|x| Cow::Owned(format!("{x:#}"))),
         SrcLang::Cpp => cpp_demangle::Symbol::new(name.as_ref())
             .ok()
-            .and_then(|x| x.demangle(&Default::default()).ok().map(Cow::Owned)),
+            .and_then(|x| x.demangle().ok().map(Cow::Owned)),
         SrcLang::Unknown => rustc_demangle::try_demangle(name.as_ref())
             .map(|x| Cow::Owned(format!("{x:#}")))
             .ok()
             .or_else(|| {
                 cpp_demangle::Symbol::new(name.as_ref())
                     .ok()
-                    .and_then(|sym| sym.demangle(&Default::default()).ok().map(Cow::Owned))
+                    .and_then(|sym| sym.demangle().ok().map(Cow::Owned))
             }),
     }
     .unwrap_or(name)
@@ -275,7 +272,10 @@ fn default_apk_dispatcher(
         info.member_mmap,
         Some(apk_elf_path.into_os_string()),
     ));
-    let resolver = ElfResolver::from_parser(parser, debug_dirs)?;
+    // TODO: Would be good to provide the `Symbolizer`'s ELF cache for
+    //       use here.
+    let elf_cache = None;
+    let resolver = ElfResolver::from_parser(parser, debug_dirs, elf_cache)?;
     let resolver = Box::new(resolver);
     Ok(resolver)
 }
@@ -317,7 +317,7 @@ pub struct Builder {
     /// List of additional directories in which split debug information
     /// is looked for.
     #[cfg(feature = "dwarf")]
-    debug_dirs: Vec<PathBuf>,
+    debug_dirs: Rc<[PathBuf]>,
     /// The "dispatch" function to use when symbolizing addresses
     /// mapping to members of an APK.
     #[cfg(feature = "apk")]
@@ -389,10 +389,7 @@ impl Builder {
                 .map(|p| p.as_ref().to_path_buf())
                 .collect();
         } else {
-            self.debug_dirs = DEFAULT_DEBUG_DIRS
-                .iter()
-                .map(PathBuf::from)
-                .collect::<Vec<_>>();
+            self.debug_dirs = DEFAULT_DEBUG_DIRS.iter().map(PathBuf::from).collect();
         }
         self
     }
@@ -454,10 +451,13 @@ impl Builder {
             elf_cache: FileCache::builder().enable_auto_reload(auto_reload).build(),
             #[cfg(feature = "gsym")]
             gsym_cache: FileCache::builder().enable_auto_reload(auto_reload).build(),
-            ksym_cache: FileCache::builder().enable_auto_reload(auto_reload).build(),
             perf_map_cache: FileCache::builder().enable_auto_reload(auto_reload).build(),
             process_vma_cache: RefCell::new(HashMap::new()),
             process_cache: InsertMap::new(),
+            #[cfg(feature = "dwarf")]
+            kernel_cache: KernelCache::new(Rc::clone(&debug_dirs)),
+            #[cfg(not(feature = "dwarf"))]
+            kernel_cache: KernelCache::new(),
             vdso_parser: OnceCell::new(),
             find_sym_opts,
             demangle,
@@ -478,10 +478,7 @@ impl Default for Builder {
             inlined_fns: true,
             demangle: true,
             #[cfg(feature = "dwarf")]
-            debug_dirs: DEFAULT_DEBUG_DIRS
-                .iter()
-                .map(PathBuf::from)
-                .collect::<Vec<_>>(),
+            debug_dirs: DEFAULT_DEBUG_DIRS.iter().map(PathBuf::from).collect(),
             #[cfg(feature = "apk")]
             apk_dispatch: None,
             process_dispatch: None,
@@ -751,7 +748,6 @@ pub struct Symbolizer {
     elf_cache: FileCache<ElfResolverData>,
     #[cfg(feature = "gsym")]
     gsym_cache: FileCache<GsymResolver<'static>>,
-    ksym_cache: FileCache<Rc<KsymResolver>>,
     perf_map_cache: FileCache<PerfMap>,
     /// Cache of VMA data on per-process basis.
     ///
@@ -759,12 +755,14 @@ pub struct Symbolizer {
     /// data by the user.
     process_vma_cache: RefCell<HashMap<Pid, Box<[maps::MapsEntry]>>>,
     process_cache: InsertMap<PathName, Option<Box<dyn Resolve>>>,
+    /// Cache of kernel related data.
+    kernel_cache: KernelCache,
     /// The ELF parser used for the system-wide vDSO.
     vdso_parser: OnceCell<Box<ElfParser<StaticMem>>>,
     find_sym_opts: FindSymOpts,
     demangle: bool,
     #[cfg(feature = "dwarf")]
-    debug_dirs: Vec<PathBuf>,
+    debug_dirs: Rc<[PathBuf]>,
     #[cfg(feature = "apk")]
     apk_dispatch: Option<Dbg<Box<dyn ApkDispatch>>>,
     process_dispatch: Option<Dbg<Box<dyn ProcessDispatch>>>,
@@ -1024,23 +1022,8 @@ impl Symbolizer {
         Ok(handler.all_symbols)
     }
 
-    fn create_ksym_resolver(&self, path: &Path, file: &File) -> Result<Rc<KsymResolver>> {
-        let resolver = KsymResolver::load_from_reader(file, path)?;
-        let resolver = Rc::new(resolver);
-        Ok(resolver)
-    }
-
-    fn ksym_resolver<'slf>(&'slf self, path: &Path) -> Result<&'slf Rc<KsymResolver>> {
-        let (file, cell) = self.ksym_cache.entry(path)?;
-        let resolver = cell.get_or_try_init_(|| self.create_ksym_resolver(path, file))?;
-        Ok(resolver)
-    }
-
     #[cfg(linux)]
-    fn create_kernel_resolver(&self, src: &Kernel) -> Result<KernelResolver> {
-        use crate::util::bytes_to_os_str;
-        use crate::MaybeDefault;
-
+    fn create_kernel_resolver<'slf>(&'slf self, src: &Kernel) -> Result<KernelResolver<'slf>> {
         let Kernel {
             kallsyms,
             vmlinux,
@@ -1049,76 +1032,17 @@ impl Symbolizer {
             _non_exhaustive: (),
         } = src;
 
-        let ksym_resolver = match kallsyms {
-            MaybeDefault::Some(kallsyms) => {
-                let ksym_resolver = self.ksym_resolver(kallsyms)?;
-                Some(ksym_resolver)
-            }
-            MaybeDefault::Default => {
-                let kallsyms = Path::new(KALLSYMS);
-                let result = self.ksym_resolver(kallsyms);
-                match result {
-                    Ok(resolver) => Some(resolver),
-                    Err(err) => {
-                        log::warn!(
-                            "failed to load kallsyms from {}: {err}; ignoring...",
-                            kallsyms.display()
-                        );
-                        None
-                    }
-                }
-            }
-            MaybeDefault::None => None,
-        };
-
-        let elf_resolver = match vmlinux {
-            MaybeDefault::Some(vmlinux) => {
-                let resolver = self
-                    .elf_cache
-                    .elf_resolver(vmlinux, self.maybe_debug_dirs(*debug_syms))?;
-                Some(resolver)
-            }
-            MaybeDefault::Default => {
-                let release = uname_release()?;
-                let release = bytes_to_os_str(release.as_bytes())?;
-                let basename = OsStr::new("vmlinux-");
-                let dirs = [Path::new("/boot/"), Path::new("/usr/lib/debug/boot/")];
-                let vmlinux = dirs.iter().find_map(|dir| {
-                    let mut file = basename.to_os_string();
-                    let () = file.push(release);
-                    let path = dir.join(file);
-                    path.exists().then_some(path)
-                });
-
-                if let Some(vmlinux) = vmlinux {
-                    let result = self
-                        .elf_cache
-                        .elf_resolver(&vmlinux, self.maybe_debug_dirs(*debug_syms));
-                    match result {
-                        Ok(resolver) => {
-                            log::debug!("found suitable vmlinux file `{}`", vmlinux.display());
-                            Some(resolver)
-                        }
-                        Err(err) => {
-                            log::warn!(
-                                "failed to load vmlinux `{}`: {err}; ignoring...",
-                                vmlinux.display()
-                            );
-                            None
-                        }
-                    }
-                } else {
-                    None
-                }
-            }
-            MaybeDefault::None => None,
-        };
-
-        KernelResolver::new(ksym_resolver.cloned(), elf_resolver.cloned(), *kaslr_offset)
+        KernelResolver::new(
+            kallsyms,
+            vmlinux,
+            *kaslr_offset,
+            *debug_syms,
+            &self.kernel_cache,
+        )
     }
 
     #[cfg(not(linux))]
-    fn create_kernel_resolver(&self, _src: &Kernel) -> Result<KernelResolver> {
+    fn create_kernel_resolver<'slf>(&'slf self, _src: &Kernel) -> Result<KernelResolver<'slf>> {
         Err(Error::with_unsupported(
             "kernel address symbolization is unsupported on operating systems other than Linux",
         ))
@@ -1177,6 +1101,7 @@ impl Symbolizer {
         &'slf self,
         src: &Source,
         input: Input<A>,
+        maybe_fold_error: fn(Error) -> Result<Symbolized<'slf>>,
     ) -> Result<A::OutTy<'slf>>
     where
         A: Copy + Addrs + 'in_,
@@ -1215,6 +1140,7 @@ impl Symbolizer {
                             None => Ok(Symbolized::Unknown(Reason::InvalidFileOffset)),
                         },
                     )
+                    .map(|result| result.or_else(maybe_fold_error))
                     .collect();
                 Ok(symbols)
             }
@@ -1243,6 +1169,7 @@ impl Symbolizer {
                     .iter()
                     .copied()
                     .map(|addr| self.symbolize_with_resolver(addr, &Resolver::Cached(resolver)))
+                    .map(|result| result.or_else(maybe_fold_error))
                     .collect();
                 Ok(symbols)
             }
@@ -1266,6 +1193,7 @@ impl Symbolizer {
                                     &Resolver::Cached(resolver.deref()),
                                 )
                             })
+                            .map(|result| result.or_else(maybe_fold_error))
                             .collect();
                         Ok(symbols)
                     }
@@ -1286,6 +1214,7 @@ impl Symbolizer {
                                     None => Ok(Symbolized::Unknown(Reason::InvalidFileOffset)),
                                 },
                             )
+                            .map(|result| result.or_else(maybe_fold_error))
                             .collect();
                         Ok(symbols)
                     }
@@ -1306,14 +1235,13 @@ impl Symbolizer {
                     }
                 };
 
-                let resolver = Rc::new(self.create_kernel_resolver(kernel)?);
+                let resolver = self.create_kernel_resolver(kernel)?;
                 let symbols = addrs
                     .as_ref()
                     .iter()
                     .copied()
-                    .map(|addr| {
-                        self.symbolize_with_resolver(addr, &Resolver::Uncached(resolver.deref()))
-                    })
+                    .map(|addr| self.symbolize_with_resolver(addr, &Resolver::Uncached(&resolver)))
+                    .map(|result| result.or_else(maybe_fold_error))
                     .collect();
                 Ok(symbols)
             }
@@ -1347,7 +1275,11 @@ impl Symbolizer {
                     *map_files,
                     *vdso,
                 )?;
-                Ok(symbols.into_iter().map(Ok).collect())
+                Ok(symbols
+                    .into_iter()
+                    .map(Ok)
+                    .map(|result| result.or_else(maybe_fold_error))
+                    .collect())
             }
             #[cfg(feature = "gsym")]
             Source::Gsym(Gsym::Data(GsymData {
@@ -1376,6 +1308,7 @@ impl Symbolizer {
                     .map(|addr| {
                         self.symbolize_with_resolver(addr, &Resolver::Uncached(resolver.deref()))
                     })
+                    .map(|result| result.or_else(maybe_fold_error))
                     .collect();
                 Ok(symbols)
             }
@@ -1404,6 +1337,7 @@ impl Symbolizer {
                     .iter()
                     .copied()
                     .map(|addr| self.symbolize_with_resolver(addr, &Resolver::Cached(resolver)))
+                    .map(|result| result.or_else(maybe_fold_error))
                     .collect();
                 Ok(symbols)
             }
@@ -1411,18 +1345,27 @@ impl Symbolizer {
         }
     }
 
-    /// Symbolize a list of addresses.
+    /// Symbolize a batch of addresses.
     ///
-    /// Symbolize a list of addresses using the provided symbolization
-    /// [`Source`].
+    /// Symbolize a batch of addresses using the provided symbolization
+    /// [`Source`]. The [`Input`] enum describes what type of address is
+    /// provided. Not all symbol sources support all address types,
+    /// e.g., due to inherent limitations of the source. If an address
+    /// type is not supported the method will return an [`Error`] of the
+    /// [`Unsupported`][ErrorKind::Unsupported] kind.
     ///
-    /// This function returns exactly one [`Symbolized`] object for each input
-    /// address, in the order of input addresses.
+    /// This method returns exactly one [`Symbolized`] object for each
+    /// input address, in the order of input addresses. Unless an error
+    /// occurs that would effect all addresses, problems preventing
+    /// symbolization of individual addresses are reported via the
+    /// [`Symbolized::Unknown`] variant. If you need more detailed error
+    /// information for such a failure, redo the operation using
+    /// [`symbolize_single`][Self::symbolize_single].
     ///
     /// The following table lists which features the various formats
-    /// (represented by the [`Source`] argument) support. If a feature is not
-    /// supported, the corresponding data in the [`Sym`] result will not be
-    /// populated.
+    /// (represented by the [`Source`] argument) support. If a feature
+    /// is not supported, the corresponding data in the [`Sym`] result
+    /// will not be populated.
     ///
     /// | Format      | Feature                          | Supported by format? | Supported by blazesym? |
     /// |-------------|----------------------------------|:--------------------:|:----------------------:|
@@ -1450,8 +1393,10 @@ impl Symbolizer {
         src: &Source,
         input: Input<&[u64]>,
     ) -> Result<Vec<Symbolized<'slf>>> {
+        let fold_error = |_err| Ok(Symbolized::Unknown(Reason::IgnoredError));
         // TODO: Use `Result::flatten` once our MSRV is 1.89.
-        self.symbolize_impl(src, input).and_then(|result| result)
+        self.symbolize_impl(src, input, fold_error)
+            .and_then(|result| result)
     }
 
     /// Symbolize a single input address/offset.
@@ -1459,6 +1404,13 @@ impl Symbolizer {
     /// In general, it is more performant to symbolize addresses in batches
     /// using [`symbolize`][Self::symbolize]. However, in cases where only a
     /// single address is available, this method provides a more convenient API.
+    ///
+    /// Note that this method also exhibits a slightly different error
+    /// reporting behavior compared to [`symbolize`][Self::symbolize]:
+    /// when symbolization of an address fails, a more comprehensive
+    /// error is reported. When possible errors are not folded into the
+    /// [`Symbolized::Unknown`] variant but conveyed directly as
+    /// [`Result::Err`].
     #[cfg_attr(feature = "tracing", crate::log::instrument(skip_all, fields(src = ?src, input = format_args!("{input:#x?}")), err))]
     pub fn symbolize_single<'slf>(
         &'slf self,
@@ -1466,7 +1418,8 @@ impl Symbolizer {
         input: Input<u64>,
     ) -> Result<Symbolized<'slf>> {
         let input = input.map(|addr| [addr; 1]);
-        self.symbolize_impl(src, input)?.0
+        let keep_error = |err| Err(err);
+        self.symbolize_impl(src, input, keep_error)?.0
     }
 
     fn maybe_debug_dirs(&self, debug_syms: bool) -> Option<&[PathBuf]> {
@@ -1491,10 +1444,14 @@ mod tests {
     use super::*;
 
     use std::env::current_exe;
+    use std::io::Write as _;
+    use std::slice;
 
+    use tempfile::NamedTempFile;
     use test_fork::fork;
     use test_log::test;
 
+    use crate::elf::types::Elf64_Ehdr;
     use crate::maps::Perm;
     use crate::symbolize::CodeInfo;
 
@@ -1512,7 +1469,9 @@ mod tests {
             .join("data")
             .join("test-stable-addrs.bin");
         let parser = Rc::new(ElfParser::open(test_elf.as_path()).unwrap());
-        let resolver = ElfResolver::from_parser(parser, None).unwrap();
+        let debug_dirs = None;
+        let elf_cache = None;
+        let resolver = ElfResolver::from_parser(parser, debug_dirs, elf_cache).unwrap();
         let resolver = Resolver::Cached(&resolver);
         assert_ne!(format!("{resolver:?}"), "");
         assert_ne!(format!("{:?}", resolver.inner()), "");
@@ -1529,7 +1488,7 @@ mod tests {
     fn symbolizer_size() {
         // TODO: This size is rather large and we should look into
         //       minimizing it.
-        assert_eq!(size_of::<Symbolizer>(), 1008);
+        assert_eq!(size_of::<Symbolizer>(), 1168);
     }
 
     /// Check that we can correctly construct the source code path to a symbol.
@@ -1724,4 +1683,78 @@ mod tests {
 
         assert_eq!(symbolizer.elf_cache.entry_count(), 1);
     }
+
+    /// Check the error reporting behavior of the
+    /// `Symbolizer::symbolize` and `Symbolizer::symbolize_single`
+    /// methods.
+    #[test]
+    fn symbolize_error_reporting() {
+        #[repr(C)]
+        struct ElfFile {
+            ehdr: Elf64_Ehdr,
+        }
+
+        let elf = ElfFile {
+            ehdr: Elf64_Ehdr {
+                e_ident: [127, 69, 76, 70, 2, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+                e_type: 3,
+                e_machine: 62,
+                e_version: 1,
+                e_entry: 4208,
+                e_phoff: size_of::<Elf64_Ehdr>() as _,
+                e_shoff: 0,
+                e_flags: 0,
+                e_ehsize: 64,
+                e_phentsize: 56,
+                e_phnum: 1000,
+                e_shentsize: 0,
+                e_shnum: 1000,
+                e_shstrndx: 0,
+            },
+        };
+
+        // Craft some broken ELF file to test error behavior.
+        let mut file = NamedTempFile::new().unwrap();
+        let dump = unsafe {
+            slice::from_raw_parts((&elf as *const ElfFile).cast::<u8>(), size_of::<ElfFile>())
+        };
+        let () = file.write_all(dump).unwrap();
+        let path = file.path();
+
+        let module = path.as_os_str().to_os_string();
+        let parser = ElfParser::from_file(file.as_file(), module.clone()).unwrap();
+        let debug_dirs = None;
+        let elf_cache = None;
+        let resolver = ElfResolver::from_parser(Rc::new(parser), debug_dirs, elf_cache).unwrap();
+        let resolver = Rc::new(resolver);
+
+        for batch in [false, true] {
+            let mut symbolizer = Symbolizer::new();
+            let () = symbolizer
+                .register_elf_resolver(path, Rc::clone(&resolver))
+                .unwrap();
+
+            let mut elf = Elf::new(path);
+            elf.debug_syms = false;
+            let src = Source::from(elf);
+            if batch {
+                let symbolized = symbolizer
+                    .symbolize(&src, Input::VirtOffset([0x1337].as_slice()))
+                    .unwrap();
+                assert_eq!(symbolized.len(), 1);
+                let symbolized = symbolized.first().unwrap();
+                let Symbolized::Unknown(reason) = symbolized else {
+                    panic!("unexpected symbolization result: {symbolized:?}");
+                };
+                assert_eq!(*reason, Reason::IgnoredError);
+            } else {
+                // `symbolize_single` is expected to report the error
+                // directly and not fold it into the `Symbolized::Unknown`
+                // variant.
+                let _err = symbolizer
+                    .symbolize_single(&src, Input::VirtOffset(0x1337))
+                    .unwrap_err();
+            }
+        }
+    }
 }
